library(cyclones)
library(replicationDemos)
library(zoo)
library(met.no.REB)

stand <- function(x,z=NULL) {
  if (is.zoo(x)) x <- coredata(x)
  if (is.null(z)) y <- (x - mean(x,na.rm=TRUE))/sd(x,na.rm=TRUE) else 
                  y <- (x - mean(z,na.rm=TRUE))/sd(z,na.rm=TRUE)
  return(y)
}

IS <- function(x) return(sum(is.finite(x)))

globave <- function(x,lon,lat) {
  gridAy <- cos(pi*lat/180)
  gridA <- rep(gridAy,length(lon))
  gridA <- gridA/sum(c(gridA))
  dim(gridA) <- c(length(lat),length(lon))
  gridA <- t(gridA)
  #print(dim(x)); print(dim(gridA)); image(gridA); addland()
  globave <- sum(gridA*x,na.rm=TRUE)
  return(globave)
}

rmoutliers <- function(x,x1=2) {
  x$xy.GCR[abs(x$xy.GCR) > x1] <- NA
  x$xy.GHG[abs(x$xy.GHG) > x1] <- NA
  x$xy.ENSO[abs(x$xy.ENSO) > x1] <- NA
  return(x)
}


glmlogit <- function(x,cal) {
  #print(summary(cal))
  cal$y <- x
  n <- length(x)
  fit <- glm(y ~ GCR + GHG + nino3.4 + vol,
             weights=rep(366,n),data=cal,family=binomial)
  #print(summary(fit))
  pre.zero <- data.frame(GCR=0,GHG=0,nino3.4=0,vol=0)
  pre.enso <- data.frame(GCR=0,GHG=0,nino3.4=1,vol=0)
  pre.ghg <- data.frame(GCR=0,GHG=1,nino3.4=0,vol=0)
  pre.gcr <- data.frame(GCR=1,GHG=0,nino3.4=0,vol=0)
  fw.res <- c(365.25*predict(fit,newdata=pre.zero),
              365.25*(predict(fit,newdata=pre.gcr)-predict(fit,newdata=pre.zero)),
              365.25*(predict(fit,newdata=pre.ghg)-predict(fit,newdata=pre.zero)),
              365.25*(predict(fit,newdata=pre.enso)-predict(fit,newdata=pre.zero)),
              fit$coefficients,mean(x))
  attr(fw.res,"description") <- c("mean","GCR","GHG","ENSO",
                        paste("coeff",1:length(fit$coefficients),"mean(x)"))
  return(fw.res)
}

regression <- function(eof,caldat,test=0) {
  xy.GCR <- eof$EOF[1,]*0; xy.GHG <- xy.GCR; xy.ENSO <- xy.GCR
  xy.VOL <- xy.GCR; 
  coefficients <- matrix(rep(0,80),4,20)
  rownames(coefficients) <- c("GCR","GHG","ENSO","Vol")
  for (i in 1:20) {
    eval(parse(text=paste("y <- caldat$Y.",i,sep="")))
    

# For testing:
    labels <- c("GCR","GHG","ENSO")
    n <- length(caldat$y);
    if (test==1) {
        caldat$GCR <- rnorm(n)
        caldat$nino3.4 <- rnorm(n)
        labels[c(1,3)] <- "noise"
    } else
    if (test==2) {
        caldat$GCR <- rnorm(n)
        caldat$GHG <- rnorm(n)
        labels[c(1,2)] <- "noise"
    } else
    if (test==3) {
        caldat$GHG <- rnorm(n)
        caldat$nino3.4 <- rnorm(n)
        labels[c(2,3)] <- "noise"
    } else
    if (test==4) {
      caldat$GHG <- caldat$year
      caldat$GCR <- rnorm(n)
      caldat$nino3.4 <- rnorm(n)
      labels[c(2,3)] <- "noise"
      labels[1] <- "trend"
    } else
    if (test==4) {
      caldat$GHG <- rnorm(n)
      caldat$GCR <- rnorm(n)
      caldat$nino3.4 <- rnorm(n)
      labels[] <- "noise"
    } else
    if (test==5) {
      caldat$GCR <- rnorm(n)
      labels[1] <- "noise"
    } 
      
    fit <- glm(y ~ GCR + GHG + nino3.4 + vol,data=caldat)
    sum <- summary(fit)
    #print(sum$coefficients)
    igcr <- (sum$coefficients[17] < 1)
    ighg <- (sum$coefficients[18] < 1)
    ienso <- (sum$coefficients[19] < 1)
    ivol <- (sum$coefficients[20] < 1)
    if (igcr) {
      xy.GCR <- xy.GCR +   fit$coefficients[2]*eof$EOF[i,]*eof$W[i]
      coefficients[1,i] <- fit$coefficients[2]
    }
    if (ighg) {
      xy.GHG <- xy.GHG +   fit$coefficients[3]*eof$EOF[i,]*eof$W[i]
      coefficients[2,i] <- fit$coefficients[3]
    }
    if (ienso) {
      xy.ENSO <- xy.ENSO + fit$coefficients[4]*eof$EOF[i,]*eof$W[i]
      coefficients[3,i] <- fit$coefficients[4]
    }
    if (ivol) {
      xy.VOL <- xy.VOL + fit$coefficients[5]*eof$EOF[i,]*eof$W[i]
      coefficients[4,i] <- fit$coefficients[5]
    }
  }
  lon <- eof$lon; lat <- eof$lat
  #print(c(length(xy.GCR),length(lat),length(lon)))
  if ( length(xy.GCR)==(length(lat)*length(lon)) ) {
    #print("gridded")
    dim(xy.GCR) <- c(length(lat),length(lon))
    dim(xy.GHG) <- c(length(lat),length(lon))
    dim(xy.ENSO) <- c(length(lat),length(lon))
    dim(xy.VOL) <- c(length(lat),length(lon))
    results <- list(xy.GCR=t(xy.GCR),xy.GHG=t(xy.GHG),xy.ENSO=t(xy.ENSO),
                    xy.VOL=t(xy.VOL),
                    lon=lon,lat=lat,coefficients=coefficients,
                    labels=labels)
  } else {
    print("scattered stations")
    results <- list(xy.GCR=xy.GCR,xy.GHG=xy.GHG,xy.ENSO=xy.ENSO,
                    xy.VOL=t(xy.VOL),
                    lon=lon,lat=lat,coefficients=coefficients,
                    labels=labels)
  }
  attr(results,'model') <- fit
  attr(results,'src') <- attr(eof,'src')
  attr(results,'param') <- attr(eof,'param')
  attr(results,'period') <- paste(min(caldat$year),max(caldat$year),sep="-")
  class(results) <- 'regression'
  invisible(results)
}

predict.regression <- function(object, ...) {
  x <- attr(object,'model')
  y <- predict(x, ...)
  return(y)
}


plotxy <- function(results) {
  #attach(results)
  lon <- results$lon; lat <- results$lat
  xy.ENSO <- results$xy.ENSO; xy.GHG <- results$xy.GHG
  xy.GCR <- results$xy.GCR
  n <- 100; wt <- pi*(1:n)/n
  col <- rgb(seq(0,1,length=n)^0.2,
             sin(wt)^0.3,
             seq(1,0,length=n)^0.2)

  mx1 <- max(abs(c(xy.ENSO)),na.rm=TRUE)
  breaks1 <- seq(-mx1,mx1,length=n+1)
  mx2 <- max(abs(c(xy.GHG)),na.rm=TRUE)
  breaks2 <- seq(-mx2,mx2,length=n+1)
  mx3 <- max(abs(c(xy.GCR)),na.rm=TRUE)
  breaks3 <- seq(-mx3,mx3,length=n+1)

  dev.new()
  par(bty="n",mfcol=c(2,2),xaxt="n",yaxt="n",xpd=FALSE,mar=c(0.5,0.5,1.5,0.5))                  
  image(lon,lat,xy.ENSO,main=results$labels[3],col=col,breaks=breaks1,xlab="",ylab="")
  addland()
  image(lon,lat,xy.GHG,main=results$labels[2],col=col,breaks=breaks2,xlab="",ylab="")
  addland()
  image(lon,lat,xy.GCR,main=results$labels[1],col=col,breaks=breaks3,xlab="",ylab="")
  addland()

  par(yaxt="s",fig=c(0.60,0.63,0.15,0.4),new=TRUE,mar=c(0,0,1.5,0),xpd=NA,cex=0.7)
  image(c(1,2),breaks1,rbind(breaks1),col=col,breaks=breaks1,xlab="",ylab="",
        main=results$labels[3])

  par(yaxt="s",fig=c(0.73,0.76,0.15,0.4),new=TRUE,mar=c(0,0,1.5,0),xpd=NA,cex=0.7)
  image(c(1,2),breaks2,rbind(breaks2),col=col,breaks=breaks2,xlab="",ylab="",
        main=results$labels[2])

  par(yaxt="s",fig=c(0.86,0.89,0.15,0.4),new=TRUE,mar=c(0,0,1.5,0),xpd=NA,cex=0.7)
  image(c(1,2),breaks3,rbind(breaks3),col=col,breaks=breaks3,xlab="",ylab="",
        main=results$labels[1])
  par(xaxt="n",yaxt="n",fig=c(0.5,1,0,0.1),new=TRUE,
      mar=c(0,0,0,0),xpd=NA,cex=0.7)
  plot(c(0,1),c(0,1),type="n",xlab="",ylab="")
  text(0.5,0.8,attr(results,'param'),cex=1.25)
  text(0.5,0.5,attr(results,'src'),cex=0.8)
  text(0.5,0.2,attr(results,'period'),cex=0.8)
}

mapstations <- function(results,xlim=c(-150,80),ylim=c(20,70)) {
  lon <- results$lon; lat <- results$lat
  xy.ENSO <- results$xy.ENSO; xy.GHG <- results$xy.GHG
  xy.GCR <- results$xy.GCR
  n <- 100; wt <- pi*(1:n)/n
  col <- rgb(seq(1,0,length=n)^0.2,
             sin(wt)^0.3,
             seq(0,1,length=n)^0.2)

  if (max(c(xy.ENSO),na.rm=TRUE)*min(c(xy.ENSO),na.rm=TRUE) < 0) {
    mx1 <- quantile(abs(c(xy.ENSO)),0.97,na.rm=TRUE)
    breaks1 <- seq(-mx1,mx1,length=n+1)
    mx2 <- quantile(abs(c(xy.GHG)),0.97,na.rm=TRUE)
    breaks2 <- seq(-mx2,mx2,length=n+1)
    mx3 <- quantile(abs(c(xy.GCR)),0.97,na.rm=TRUE)
    breaks3 <- seq(-mx3,mx3,length=n+1)
  } else {
    mx1 <- quantile(abs(c(xy.ENSO)),0.97,na.rm=TRUE)
    breaks1 <- seq(0,mx1,length=n+1)
    mx2 <- quantile(abs(c(xy.GHG)),0.97,na.rm=TRUE)
    breaks2 <- seq(0,mx2,length=n+1)
    mx3 <- quantile(abs(c(xy.GCR)),0.97,na.rm=TRUE)
    breaks3 <- seq(0,mx3,length=n+1)
  }

  cols1 <- rep(rgb(1,1,1),length(xy.ENSO)); cols2 <- cols1; cols3 <- cols1
  for (i in 1:length(xy.ENSO)) {
    ienso <- min(max(sum(xy.ENSO[i] > breaks1),1),n)
    ighg <- min(max(sum(xy.GHG[i] > breaks2),1),n)
    igcr <- min(max(sum(xy.GCR[i] > breaks3),1),n)
    
    if (is.finite(ienso)) cols1[i] <- col[ienso]
    if (is.finite(ighg)) cols2[i] <- col[ighg]
    if (is.finite(igcr)) cols3[i] <- col[igcr]
  }
  
  dev.new()
  par(bty="n",mfcol=c(2,2),xaxt="n",yaxt="n",
      xpd=FALSE,mar=c(0.5,0.5,1.5,0.5))                  
  plot(lon,lat,main=results$labels[3],
       col=cols1,pch=19,cex=0.4,xlab="",ylab="",
       xlim=xlim,ylim=ylim)
  addland()
  plot(lon,lat,main=results$labels[2],
       col=cols2,pch=19,cex=0.4,xlab="",ylab="",
       xlim=xlim,ylim=ylim)
  addland()
  plot(lon,lat,main=results$labels[1],
       col=cols3,pch=19,cex=0.4,xlab="",ylab="",
       xlim=xlim,ylim=ylim)
  addland()

  par(yaxt="s",fig=c(0.60,0.63,0.15,0.4),new=TRUE,mar=c(0,0,1.5,0),xpd=NA,cex=0.7)
  image(c(1,2),breaks1,rbind(breaks1),col=col,breaks=breaks1,xlab="",ylab="",
        main=results$labels[3])

  par(yaxt="s",fig=c(0.73,0.76,0.15,0.4),new=TRUE,mar=c(0,0,1.5,0),xpd=NA,cex=0.7)
  image(c(1,2),breaks2,rbind(breaks2),col=col,breaks=breaks2,xlab="",ylab="",
        main=results$labels[2])

  par(yaxt="s",fig=c(0.86,0.89,0.15,0.4),new=TRUE,mar=c(0,0,1.5,0),xpd=NA,cex=0.7)
  image(c(1,2),breaks3,rbind(breaks3),col=col,breaks=breaks3,xlab="",ylab="",
        main=results$labels[1])

  par(xaxt="n",yaxt="n",fig=c(0.5,1,0,0.1),new=TRUE,
      mar=c(0,0,0,0),xpd=NA,cex=0.7)
  plot(c(0,1),c(0,1),type="n",xlab="",ylab="")
  text(0.5,0.8,attr(results,'param'),cex=1.25)
  text(0.5,0.5,attr(results,'src'),cex=1.5)
  text(0.5,0.2,attr(results,'period'),cex=0.8)
}

#----------------------------------------------------------------------------------

# source("prep.R")

#---------------------------------------------------------------------
# Get the EOFs of annuam meal T(2m) and SLP from ameofs.R
load("hepclimeff.rda")
load("ameofs.rda")

cal.period <- 1951:2000

GMT <- GMT - mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])
gmt <- gmt - mean(gmt[is.element(as.numeric(index(gmt)),1961:1990)])

mu.clim <- rowMeans(mu)
mu <- mu - mu.clim
pca <- svd(mu)
eof.mu <- list(EOF=t(pca$u[,1:30]),W=pca$d[1:30],PC=t(pca$v[,1:30]),lon=attr(mu,'lon'),
               lat=attr(mu,'lat'))
#plot(cumsum(100*(pca$d^2)/sum(pca$d^2))); dev.new()

i1 <- is.element(eof.t2m$yy,cal.period)
j1 <- is.element(eof.gisstemp$yy,cal.period)
k1 <- is.element(eof.prmsl$yy,cal.period)
l1 <- is.element(eof.slp$yy,cal.period)
i2 <- is.element(forcings$Year,cal.period)
I2 <- is.element(forcings$Year,1951:2000)
i3 <- is.element(as.numeric(index(Y$NINO3.4)),cal.period)
I3 <- is.element(as.numeric(index(Y$NINO3.4)),1951:2000)
# Transform ENSO-index (non-linear aspect):

prep <- data.frame(GMT=GMT[is.element(as.numeric(index(GMT)),cal.period)],
                   ENSO=Y$NINO3.4[i3],t=cal.period)
trend <- glm(GMT ~ I(t) + I(t^2) + I(t^3),data=prep)
prep$detrend=trend$residual
ensofit <- glm(detrend ~ I(ENSO) + I(ENSO^2) + I(ENSO^3),data=prep)
print(summary(ensofit))
nino3.4 <- data.frame(ENSO=Y$NINO3.4)
#ENSO <- predict(ensofit,newdata=nino3.4)
ENSO <- Y$NINO3.4

plot(trend$residual,ENSO[i3])

cal.t2m <- data.frame(Y=eof.t2m$PC[i1,],
                      GCR=stand(forcings$gcr[i2]),
                      GHG=stand(forcings$W.M_GHGs[i2]),
                      nino3.4=stand(ENSO[i3]),
                      vol=stand(forcings$StratAer[i2]),
                      sol=stand(forcings$Solar[i2]),
                      year=eof.t2m$yy[i1])
cal.T2m <- data.frame(Y=eof.T2m$PC[i1,],
                      GCR=stand(forcings$gcr[i2]),
                      GHG=stand(forcings$W.M_GHGs[i2]),
                      nino3.4=stand(ENSO[i3]),
                      vol=stand(forcings$StratAer[i2]),
                      sol=stand(forcings$Solar[i2]),
                      year=eof.T2m$yy[i1])
Cal.t2m <- data.frame(Y=eof.gisstemp$PC[j1,],
                      GCR=stand(forcings$gcr[i2]),
                      GHG=stand(forcings$W.M_GHGs[i2]),
                      nino3.4=stand(ENSO[i3]),
                      vol=stand(forcings$StratAer[i2]),
                      sol=stand(forcings$Solar[i2]),
                      year=eof.gisstemp$yy[j1])

cal.slp <- data.frame(Y=eof.prmsl$PC[k1,],
                      GCR=stand(forcings$gcr[i2]),
                      GHG=stand(forcings$W.M_GHGs[i2]),
                      nino3.4=stand(ENSO[i3]),
                      vol=stand(forcings$StratAer[i2]),
                      sol=stand(forcings$Solar[i2]),
                      year=eof.prmsl$yy[k1])
Cal.slp <- data.frame(Y=eof.slp$PC[l1,],
                      GCR=stand(forcings$gcr[i2]),
                      GHG=stand(forcings$W.M_GHGs[i2]),
                      nino3.4=stand(ENSO[i3]),
                      vol=stand(forcings$StratAer[i2]),
                      sol=stand(forcings$Solar[i2]),
                      year=eof.slp$yy[l1])
cal.mu <- data.frame(Y=t(pca$v),
                     GCR=stand(forcings$gcr[I2]),
                     GHG=stand(forcings$W.M_GHGs[I2]),
                     nino3.4=stand(ENSO[i3]),
                     vol=stand(forcings$StratAer[i2]),
                     sol=stand(forcings$Solar[i2]),
                     year=forcings$Year[I2])

cal    <- data.frame(GCR=stand(forcings$gcr[I2]),
                     GHG=stand(forcings$W.M_GHGs[I2]),
                     nino3.4=stand(ENSO[I3]),
                     vol=stand(forcings$StratAer[I2]),
                     sol=stand(forcings$Solar[I2]),
                     year=forcings$Year[I2])
Cal    <- data.frame(GCR=stand(forcings$gcr),
                     GHG=stand(forcings$W.M_GHGs),
                     nino3.4=stand(ENSO),
                     vol=stand(forcings$StratAer),
                     sol=stand(forcings$Solar),
                     year=forcings$Year)

calzoo <- zoo(cal.t2m,order.by=as.character(1955:2004))

regression(eof.t2m,cal.t2m) -> results.t2m
rmoutliers(results.t2m) -> results.t2m
plotxy(results.t2m)
dev.copy2eps(file="erlfig1.eps")
results.t2m$coefficients

regression(eof.gisstemp,Cal.t2m) -> results.T2m
plotxy(results.T2m)
dev.copy2eps(file="erlfigS1.eps")

regression(eof.prmsl,cal.slp) -> results.slp
plotxy(results.slp)
dev.copy2eps(file="erlfigS2a.eps")

regression(eof.slp,Cal.slp) -> results.Slp
plotxy(results.Slp)
dev.copy2eps(file="erlfigS2b.eps")

regression(eof.mu,cal.mu) -> results.mu
results.mu$labels=c("GCR","GHG","ENSO")
attr(results.mu,'param') <- expression(mu)
attr(results.mu,'src') <- 'GDCN'
mapstations(results.mu)
dev.copy2eps(file="erlfigS3.eps")

results.vol <- results.T2m
results.vol$xy.ENSO <- results.vol$xy.VOL
results.vol$labels=c("GCR","GHG","Vol")
plotxy(results.vol)
dev.copy2eps(file="erlfigS9.eps")

print("fw:")
res.fw <- apply(fw,1,glmlogit,cal)
results.fw <- list(xy.GCR=res.fw[2,],xy.GHG=res.fw[3,],xy.ENSO=res.fw[4,],
                   lon=attr(fw,'lon'),lat=attr(fw,'lat'),
                   coefficients=res.fw[5:7,],labels=c("GCR","GHG","ENSO"))
attr(results.fw,'param') <- expression(f[w])
attr(results.fw,'src') <- 'GDCN'
attr(results.fw,'period') <- '1951-2000'
mapstations(results.fw)
dev.copy2eps(file="erlfigS4.eps")


breaks <- seq(-1,2,by=0.1)
h.ghg <- hist(results.t2m$xy.GHG,plot=FALSE,breaks=breaks)
h.gcr <- hist(results.t2m$xy.GCR,plot=FALSE,breaks=breaks)
h.enso <- hist(results.t2m$xy.ENSO,plot=FALSE,breaks=breaks)
dev.new()
par(bty="n",xaxt="s",yaxt="n",xpd=NA,mar=c(6.5,3.5,1.5,0.5)) 
plot(c(0,0),c(0,10),lwd=3,type="l",col="grey",
     xlab="",ylab="",xlim=c(-1,2),ylim=c(0,7))
lines(h.ghg$mids,h.ghg$density,lwd=3,col="darkgreen")
lines(h.gcr$mids,h.gcr$density,lwd=3,col="red")
lines(h.enso$mids,h.enso$density,lwd=3,col="blue")
legend(-0.75,-1,c("GCR","GHG","ENSO"),col=c("red","darkgreen","blue"),
       bty="n",horiz = TRUE,lty=1,lwd=3)
dev.copy2eps(file="erlfigS5.eps")

# Use the weights to regression weights and EOF patterns compute global mean temperature values...

# GMT...
# GCR*xy.GCR
# GHG*xy.GHG
# NINO3.4*xy.ENSO

print("predict back in time:")
regression(eof.T2m,cal.T2m) -> results.T2m

# Monte-Carlo simulations
N <- 300

for (i in 1:N) {
  regression(eof.T2m,cal.T2m,test=5) -> test
  x.gcr <- globave(test$xy.GCR,test$lon,test$lat) *
         stand(forcings$gcr[i2])
  x.gcr <- x.gcr - mean(x.gcr[is.element(forcings$Year[i2],1961:1990)]) +
         mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])   
  gcr <- zoo(x.gcr,order.by=forcings$Year[i2])
  if (i==1) gcr.mc <- gcr else
            gcr.mc <- merge(gcr.mc,gcr)
}

extrap <- predict(results.T2m,newdata=Cal)

x.ghg <- globave(results.T2m$xy.GHG,results.T2m$lon,results.T2m$lat) *
         stand(forcings$W.M_GHGs[i2])
x.ghg <- x.ghg - mean(x.ghg[is.element(forcings$Year[i2],1961:1990)]) +
         mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])   
ghg <- zoo(x.ghg,order.by=forcings$Year[i2])
x.GHG <- globave(results.T2m$xy.GHG,results.T2m$lon,results.T2m$lat) *
         stand(forcings$W.M_GHGs,forcings$W.M_GHGs[i2])
x.GHG <- x.GHG - mean(x.GHG[is.element(forcings$Year,1961:1990)]) +
         mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])   
GHG <- zoo(x.GHG,order.by=forcings$Year)
x.gcr <- globave(results.T2m$xy.GCR,results.T2m$lon,results.T2m$lat) *
         stand(forcings$gcr[i2])
x.gcr <- x.gcr - mean(x.gcr[is.element(forcings$Year[i2],1961:1990)]) +
         mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])   
gcr <- zoo(x.gcr,order.by=forcings$Year[i2])
x.GCR <- globave(results.T2m$xy.GCR,results.T2m$lon,results.T2m$lat) *
         stand(forcings$gcr,forcings$gcr[i2])
x.GCR <- x.GCR - mean(x.GCR[is.element(forcings$Year,1961:1990)]) +
         mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])   
GCR <- zoo(x.GCR,order.by=forcings$Year)
x.enso <- globave(results.T2m$xy.ENSO,results.T2m$lon,results.T2m$lat) *
         stand(ENSO[i2])
x.enso <- x.enso - mean(x.enso[is.element(forcings$Year[i2],1961:1990)]) +
         mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])   
enso <- zoo(x.enso,order.by=forcings$Year[i2])
x.ENSO <- globave(results.T2m$xy.ENSO,results.T2m$lon,results.T2m$lat) *
         stand(ENSO,ENSO[i2])
x.ENSO <- x.ENSO - mean(x.ENSO[is.element(forcings$Year,1961:1990)]) +
         mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])   
ENSO <- zoo(x.ENSO,order.by=forcings$Year)
x.VOL <- globave(results.T2m$xy.VOL,results.T2m$lon,results.T2m$lat) *
         stand(forcings$StratAer,forcings$StratAer[i2])
x.VOL <- x.VOL - mean(x.VOL[is.element(forcings$Year,1961:1990)]) +
         mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])   
VOL <- zoo(x.VOL,order.by=forcings$Year)
x.all <- x.ghg + x.gcr + x.enso
x.all <- x.all - mean(x.all[is.element(forcings$Year[i2],1961:1990)]) +
       mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])
all <- zoo(x.all,order.by=forcings$Year[i2])
x.ALL <- x.GHG + x.GCR + x.ENSO + x.VOL
x.ALL <- x.ALL - mean(x.ALL[is.element(forcings$Year,1961:1990)]) +
                 mean(GMT[is.element(as.numeric(index(GMT)),1961:1990)])
ALL <- zoo(x.ALL,order.by=forcings$Year)

res <- GMT - ALL

trend.obs <- lm(GMT[i2] ~ forcings$Year[i2])
trend.ghg <- lm(x.ghg ~ forcings$Year[i2])
trend.gcr <- lm(x.gcr ~ forcings$Year[i2])
trend.enso <- lm(x.enso ~ forcings$Year[i2])
trend.forc <- lm(forcings$W.M_GHGs[i2] ~ forcings$Year[i2])
sfit.ghg <- summary(trend.ghg)
sfit.gcr <- summary(trend.gcr)
sfit.enso <- summary(trend.enso)
sfit.forc <- summary(trend.forc)
sfit.obs <- summary(trend.obs)
slope.obs <- paste(round(10*sfit.obs$coefficients[2],3),"%+-%",
                   round(10*sfit.obs$coefficients[4],3),"*degree*C/d*e*c*a*d*e")
slope.ghg <- paste(round(10*sfit.ghg$coefficients[2],3),"%+-%",
                   round(10*sfit.ghg$coefficients[4],3),"*degree*C/d*e*c*a*d*e")
slope.gcr <- paste(round(10*sfit.gcr$coefficients[2],3),"%+-%",
                   round(10*sfit.gcr$coefficients[4],3),"*degree*C/d*e*c*a*d*e")
slope.enso <- paste(round(10*sfit.enso$coefficients[2],3),"%+-%",
                    round(10*sfit.enso$coefficients[4],3),"*degree*C/d*e*c*a*d*e")
legOBS <- eval(parse(text=paste("expression(",slope.obs,")")))
legGHG <- eval(parse(text=paste("expression(",slope.ghg,")")))
legGCR <- eval(parse(text=paste("expression(",slope.gcr,")")))
legENSO <- eval(parse(text=paste("expression(",slope.enso,")")))

errest <- sqrt((sfit.ghg$coefficients[4]/sfit.ghg$coefficients[2])^2 +
               (sfit.forc$coefficients[4]/sfit.forc$coefficients[2])^2)*
          sfit.ghg$coefficients[2]
transclimsens <- paste(round(sfit.ghg$coefficients[2]/sfit.forc$coefficients[2],2),"%+-%",round(errest,2))


dev.new()
par(bty="n",xaxt="s",yaxt="s",xpd=FALSE,mar=c(3.5,4.5,1.5,1),fig=c(0,1,0.05,1),
    col.axix="grey") 
plot(GMT,xlim=c(1900,2010),ylim=range(GMT,gmt,na.rm=TRUE),lwd=4,
     main="Global mean temperature",xlab="",
     ylab=expression(paste("temperature ",degree*C)))
for (i in 1:N) lines(gcr.mc[,i],col="pink",lty=3)
lines(GMT,lwd=4)
lines(gmt,col="grey30",lwd=3)
lines(ALL,col="grey",lwd=5)
lines(ghg,col="darkgreen",lwd=3)
lines(GHG,col="darkgreen",lwd=2,lty=2)
lines(gcr,col="red",lwd=3)
lines(GCR,col="red",lwd=2,lty=2)
lines(enso,col="blue",lwd=3)
lines(ENSO,col="blue",lwd=2,lty=2)
abline(trend.ghg,col="darkgreen",lty=2)
abline(trend.gcr,col="red",lty=2)
abline(trend.enso,col="blue",lty=2)
abline(trend.obs,lty=2)

legend(1905,0.6,c("Trends:",legOBS,legGHG,legGCR,legENSO),cex=0.8,
       text.col=c("grey30","black","darkgreen","red","blue"),bty="n")

legsens <- eval(parse(text=paste("expression(paste(",transclimsens,
                        ", degree, 'C/',(W*m^2)))")))
text(1980,-0.6,legsens,pos=4,cex=0.8)

par(xaxt="n",yaxt="n",bty="n",fig=c(0,1,0,0.1),new=TRUE,mar=c(0,0,0,0),cex=0.7)
plot(c(0,1),c(0,1),xlab="",ylab="",type="n")
legend(0.1,0.5,c("GISSTEMP","20C reanalysis","GCR","GHG","ENSO","All"),
       col=c("black","grey30","red","darkgreen","blue","grey"),
       bty="n",horiz = TRUE,lty=1,lwd=c(4,rep(3,5)),cex=1)
dev.copy2eps(file="erlfig2.eps")



dev.new()
par(bty="n",xaxt="s",yaxt="s",xpd=FALSE,mar=c(3.5,4.5,1.5,1),fig=c(0,1,0.05,1),
    col.axix="grey") 
plot(GMT,xlim=c(1900,2010),ylim=range(GMT,gmt,na.rm=TRUE),lwd=4,
     main="Global mean temperature",xlab="",
     ylab=expression(paste("temperature ",degree*C)))
lines(GMT,lwd=4)
lines(gmt,col="grey30",lwd=3)
lines(VOL,col="cyan",lwd=3)
lines(GCR,col="red",lwd=2,lty=2)

par(xaxt="n",yaxt="n",bty="n",fig=c(0,1,0,0.1),new=TRUE,mar=c(0,0,0,0),cex=0.7)
plot(c(0,1),c(0,1),xlab="",ylab="",type="n")
legend(0.1,0.5,c("GISSTEMP","20C reanalysis","GCR","Volcanos","Solar"),
       col=c("black","grey30","red","cyan","magenta"),
       bty="n",horiz = TRUE,lty=1,lwd=c(4,rep(3,4)),cex=1)
dev.copy2eps(file="erlfigS10.eps")


dev.new()
par(bty="n")
plot(res,lwd=3,main="GISSTEMP - contribution from covariates",xlab="")
lines(res*0,col="grey",lty=3)
dev.copy2eps(file="erlfigS6.eps")

dev.new()
par(bty="n")
ccf(GMT,GCR,lwd=3, lag.max=10)
grid()
dev.copy2eps(file="erlfigS7.eps")


par(bty="n")
s <- spectrum(res,plot=FALSE)
plot(1/s$freq,s$spec,
     xlim=c(-5,135),type="l",lwd=3,xlab="time scale",ylab="spectral power")
grid(nx=30)
grid(nx=15,lty=1)
lines(1/s$freq,s$spec,lwd=3)
dev.copy2eps(file="erlfigS8.eps")

dev.new()
hist(100*apply(mu,1,sd)/mu.clim,breaks=seq(0,200,by=1),xlim=c(0,100))
grid()
dev.copy2eps(file="erlfigS11.eps")

dev.new()
par(xaxt="s",yaxt="s",bty="n",fig=c(0,1,0.05,1))
plot(stand(coredata(res)),stand(coredata(ALL)),pch=19,col="grey",cex=1.2,
     main="Check for covariance",xlab="residual T",
     ylab="covariates")
points(stand(coredata(res)),stand(coredata(ENSO)),col="blue",lwd=2,cex=0.7)
points(stand(coredata(res)),stand(coredata(GHG)),col="darkgreen",pch=2,lwd=2)
points(stand(coredata(res)),stand(coredata(GCR)),col="red",pch=3,lwd=2)
points(stand(coredata(res)),stand(coredata(VOL)),col="cyan",pch=4,lwd=2,cex=0.7)
par(xaxt="n",yaxt="n",bty="n",fig=c(0,1,0.0,.05),mar=rep(0,4),new=TRUE)
plot(c(0,1),c(0,1),type="n",xlab="",ylab="")
legend(0.1,0.95,c("All","ENSO","GHG","GCR","Vol"),pch=c(19,21,2,3,4),
       col=c("grey","blue","darkgreen","red","cyan"),bty="n",horiz=TRUE)
dev.copy2eps(file="erlfigS12.eps")

# Project the residual onto EOFs for temperature: the AMO?
l1 <- is.element(as.numeric(index(res)),eof.t2m$yy)
l2 <- is.element(eof.t2m$yy,as.numeric(index(res)))
res.mod <- data.frame(y=coredata(res[l1]),x=eof.t2m$PC[l2,])
resfit <- glm(y ~ x.1 + x.2 + x.3 + x.4 + x.5 + x.6 +
              x.7 + x.8 + x.9 + x.10 + x.11 + x.12 +
              x.13 + x.14 + x.15 + x.16 + x.17 + x.18 +
              x.19 + x.20,data=res.mod)
res.t2m <- zoo(predict(resfit,newdata=res.mod),order.by=eof.t2m$yy[l2,])
dev.new()
plot(res,lwd=4)
lines(res.t2m,col="red",lwd=2)
dev.copy2eps(file="erlfigS13.eps")

# Plot the map showing regional response
alpha <- resfit$coefficients[2:21]
X.res <- eof.t2m$EOF[i,]*0
for (i in 1:20) X.res <- X.res + eof.t2m$W[i] * alpha[i] * eof.t2m$EOF[i,]

n <- 100; wt <- pi*(1:n)/n
col <- rgb(seq(0,1,length=n)^0.3 + sin(wt)^0.5,
           sin(wt)^0.5,
           seq(1,0,length=n)^0.3 + sin(wt)^0.5)

mx1 <- max(abs(c(X.res)),na.rm=TRUE)
breaks1 <- seq(-mx1,mx1,length=n+1)
dim(X.res) <- c(length(eof.t2m$lat),length(eof.t2m$lon))

dev.new()
par(bty="n",mfcol=c(1,1),xaxt="n",yaxt="n",
    xpd=FALSE,mar=c(0.5,0.5,1.5,0.5))                  
image(eof.t2m$lon,eof.t2m$lat,t(X.res),
      main="T(2m) response associated with residual",
      col=col,breaks=breaks1,xlab="",ylab="")
addland()

par(yaxt="s",fig=c(0.05,0.08,0.05,0.25),new=TRUE,
    mar=c(0,0,1.5,0),xpd=NA,cex=0.7)
image(c(1,2),breaks1,rbind(breaks1),col=col,breaks=breaks1,xlab="",ylab="")

dev.copy2eps(file="erlfigS14.eps")
                      
