
  # Prepare the data:
data(gcr)
data(forcings)

Rz <- Sunspots()
yr <- trunc(Rz$year)
mo <- Rz$yyyymm - yr*100
dy <- rep(1,length(mo))
ss <- zoo(Rz$sunspotnumber,order.by=as.Date(paste(yr,mo,dy,sep="-")))
SS <- aggregate(ss,by=as.character(yr))

Date <- as.character(gcr$Date)
Date <- paste(substr(Date,7,10),substr(Date,1,2),substr(Date,4,5),sep="-")
climax <- zoo(as.numeric(as.character(gcr$Climax)),order.by=as.Date(Date))

good <- is.finite(climax)
climax <- climax[good]
year <- format(index(climax),'%Y')
Climax <- aggregate(climax,by=year)

Z <- merge(SS,Climax,all=FALSE)
ssgcr <- data.frame(y=Z$Climax,x=Z$SS)

gcrfit <- glm(y ~ x, data=ssgcr)
pred <- zoo(predict(gcrfit),order.by=index(Z))
plot(Climax)
lines(pred,col="red")

proj <- data.frame(x=SS)
gcr.x <- zoo(predict(gcrfit,new.data=proj),order.by=index(SS))
plot(gcr.x)
lines(Climax,col="red")
ii <- is.element(index(SS),index(Climax))
gcr <- gcr.x
gcr[ii] <- Climax
W <- zoo(rep(round(sd(gcr.x)/sd(Climax),1),length(gcr.x)),index(SS))
W[ii] <- 1

Sol <- zoo(forcings$Solar,order.by=as.character(forcings$Year))
GCR <- merge(Sol,gcr,all=FALSE)
ww <- merge(Sol,W,all=FALSE)

forcings$gcr <- GCR$gcr
forcings$W <- ww$W

enso <- ENSO()
nino3.4 <- zoo(enso$nino3.4.3month,order.by=as.Date(paste(enso$year,enso$month,"1",sep="-")))
NINO3.4 <- aggregate(nino3.4,by=as.character(enso$year))

Y <- merge(Sol,NINO3.4,all=FALSE)

t2m <- retrieve.nc("air.sig995.an.mean.nc",
                   v.nam="air",silent=TRUE,l.scal=FALSE)
globalMean(t2m) -> gmt
gmt <- zoo(gmt,order.by=as.character(t2m$yy))

T2M <- retrieve.nc("air.2x2.1200.an.anom.comb.nc",
                   v.nam="air",silent=TRUE,l.scal=FALSE)
globalMean(T2M) -> GMT
GMT <- zoo(GMT,order.by=as.character(T2M$yy))

load("qqt1yr.world.rda")
ii <- is.element(qqt1yr.world$years,1951:2000)
mu <- qqt1yr.world$MU[,ii]
fw <- qqt1yr.world$FW[,ii]
ngood <- apply(qqt1yr.world$MU[,ii],1,IS)
test <- list(xy.GCR=rowMeans(mu,na.rm=TRUE),xy.GHG=rowMeans(fw,na.rm=TRUE),
             xy.ENSO=ngood,lon=qqt1yr.world$lon,
             lat=qqt1yr.world$lat,coefficients=NULL,
             labels=c(expression("mu"),expression("f[w]"),expression("n")))
good <- is.finite(rowMeans(mu))
mu <- mu[good,]
fw <- fw[good,]
attr(mu,'lon') <- qqt1yr.world$lon[good]
attr(mu,'lat') <- qqt1yr.world$lat[good]
plot(attr(mu,'lon'),attr(mu,'lat')); addland()
attr(mu,'alt') <- qqt1yr.world$alt[good]
attr(mu,'stnr') <- qqt1yr.world$stnr[good]
attr(mu,'year') <- qqt1yr.world$years[ii]
attr(fw,'lon') <- attr(mu,'lon'); attr(fw,'lat') <- attr(mu,'lat')
attr(fw,'alt') <- attr(mu,'alt'); attr(fw,'stnr') <- attr(mu,'stnr')
attr(fw,'year') <- attr(mu,'year')
mapstations(test)
save(file="hepclimeff.rda",forcings,Y,gmt,GMT,mu,fw,test)

